/*****************************************************************************
 * Midi Notes
 * -----------
 *
 *    Midi Number   Note Name
 *    -----------   ---------
 *    21            A0
 *    22            A#0 or Bb0
 *    23            B0
 *    24            C1
 *    25            C#1 or Db1
 *    26            D1
 *    27            D#1 or Eb1
 *    28            E1
 *    29            F1
 *    30            F#1 or Gb1
 *    31            G1
 *    32            G#1 or Ab2
 *    33            A1
 *    ...           ...
 *    95            B6
 *    96            C7
 *    97            C#7 or Db7
 *    98            D7
 *    99            D#7 or Eb7
 *    100           E7
 *    101           F7
 *    102           F#7 or Gb7
 *    103           G7
 *    104           G#7 or Ab7
 *    105           A7
 *    106           A#7 or Bb7
 *    107           B7
 *    108           C8
 *
 * Guitar Equivalents
 * ------------------
 *
 *    String   Midi Number   Hz
 *    ------   -----------   --
 *     E        40            82.4
 *     A        45           110.0
 *     D        50           146.8
 *     G        55           196.0
 *     B        59           246.9
 *     E        64           329.6
 *
 *****************************************************************************/

#ifndef _MIDI_H_
#   define _MIDI_H_

#include <string>

#include <windows.h>
#include <mmsystem.h>

using std::string;

#define MIDI_MIN_VALUE (1)
#define MIDI_MAX_VALUE (128)

#define MIDI_MIN_INSTRUMENT MIDI_MIN_VALUE
#define MIDI_MAX_INSTRUMENT MIDI_MAX_VALUE

#define MIDI_MIN_NOTE       MIDI_MIN_VALUE
#define MIDI_MAX_NOTE       MIDI_MAX_VALUE

#define MIDI_MIN_VOLUME     MIDI_MIN_VALUE
#define MIDI_MAX_VOLUME     MIDI_MAX_VALUE

class Midi
{
   public:
      static Midi *make(int port, string &errMsg);
      ~Midi();
      void setInstrument(int instrument);

      bool doCommand(
         string &errMsg,
         int noteNumber,
         int volume,
         int midiChannelNumber
      );

      void playNote(
         int noteNumber,
         int millis,
         int volume,
         int midiChannelNumber
      );

      void note(
         int noteNumber,
         int volume,
         int midiChannelNumber
      );

      static int instrumentNameToNumber(const string &name);
      static string instrumentNumberToName(int number);

   private:
      Midi(HMIDIOUT device);
      Midi(const Midi &);
      Midi &operator=(const Midi &);
      

   private:
      HMIDIOUT mDevice;
      int      mCurrentInstrument;
};

#endif
